from .tables.outpost import Outpost
from .tables.reference import Reference
from .tables.exchange import Exchange
from .tables.route import Route
from .tables.producer import Producer
from .tables.queue import Queue



from drf_spectacular.utils import OpenApiExample, OpenApiTypes, OpenApiParameter


class ReferenceV1Doc:
    @staticmethod
    def modelviewset_list_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.QUERY,
                name="page",
                required=False,
                type=OpenApiTypes.INT,
                examples=[
                    OpenApiExample("Example 1", summary="Pagination", value=1),
                    OpenApiExample("Example 2", summary="Pagination", value=2),
                ],
            )
        ]

    @staticmethod
    def modelviewset_list_examples():
        return [
            OpenApiExample(
                "Reference - List All",
                value=[
                    {
                        "identifier": "ORG=Kubefacets",
                        "type": "ORG",
                        "attrs": {"name": "Kubefacets Inc", "slug": "kubefacets-inc"},
                    }
                ],
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_get_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "Reference - Get",
                        summary="Reference Identifier",
                        description="Reference - Get",
                        value="ORG=Kubefacets",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_get_examples():
        return [
            OpenApiExample(
                "Reference - Get",
                value={
                    "identifier": "ORG=Kubefacets",
                    "type": "ORG",
                    "attrs": {"name": "Kubefacets Inc", "slug": "kubefacets-inc"},
                },
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_create_examples():
        return [
            OpenApiExample(
                "Reference - Create",
                value={
                    "identifier": "ORG=Kubefacets",
                    "type": "ORG",
                    "attrs": {"name": "Kubefacets Inc", "slug": "kubefacets-inc"},
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "Reference - Create",
                value={
                    "identifier": "ORG=Kubefacets",
                    "type": "ORG",
                    "attrs": {"name": "Kubefacets Inc", "slug": "kubefacets-inc"},
                },
                request_only=False,
                response_only=True,
            ),
        ]

    @staticmethod
    def modelviewset_delete_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "Reference - Delete",
                        summary="Reference Identifier",
                        description="Reference - Delete",
                        value="ORG=Kubefacets",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "Reference - Patch",
                        summary="Reference Identifier",
                        description="Reference - Patch",
                        value="ORG=Kubefacets",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_examples():
        return [
            OpenApiExample(
                "Reference - Patch",
                value={
                    "attrs": {"name": "Kubefacets", "slug": "kubefacets"},
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "Reference - Patch",
                value={
                    "identifier": "ORG=Kubefacets",
                    "type": "ORG",
                    "attrs": {"name": "Kubefacets", "slug": "kubefacets"},
                },
                request_only=False,
                response_only=True,
            ),
        ]



from rest_framework import serializers
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
)

from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH
from kfsd.apps.endpoints.serializers.common.model import BaseModelSerializer
from kfsd.apps.models.tables.reference import Reference


class ReferenceModelSerializer(BaseModelSerializer):
    identifier = serializers.CharField(read_only=False)
    type = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
        ]
    )
    attrs = serializers.JSONField(default=dict)

    class Meta:
        model = Reference
        fields = "__all__"


class ReferenceViewModelSerializer(BaseModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = Reference
        exclude = ("created", "updated", "id")



from django.core.management.base import BaseCommand
from kfsd.apps.core.msmq.rabbitmq.base import RabbitMQ
from kfsd.apps.core.utils.time import Time
from kfsd.apps.endpoints.serializers.common.outpost import MsgSerializer

from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.models.tables.outpost import Outpost, send_msg
from kfsd.apps.models.constants import INBOUND_SIGNAL_CLEAR_OUTPOST


import json

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


def clear_outpost(data):
    outpostQS = Outpost.objects.filter(status="E").order_by("created")
    identifiers = [outpost.identifier for outpost in outpostQS]
    if not identifiers:
        logger.info(
            "[ ACTION: INBOUND_SIGNAL_CLEAR_OUTPOST ] Outpost tbl has no messages with error status"
        )
    else:
        logger.info(
            "[ ACTION: INBOUND_SIGNAL_CLEAR_OUTPOST ] identifiers: {} with 'Error' status found".format(
                identifiers
            )
        )

    for outpostIns in outpostQS:
        send_msg(outpostIns)


callback_map = {INBOUND_SIGNAL_CLEAR_OUTPOST: clear_outpost}


def base_callback(ch, method, properties, body):
    bodyStr = body.decode().replace("'", '"')
    jsonStr = json.loads(bodyStr)
    serializedData = MsgSerializer(data=jsonStr)
    serializedData.is_valid(raise_exception=True)

    action = DictUtils.get(serializedData.data, "action")
    if action in callback_map:
        callback_map[action](serializedData.data)
    else:
        logger.error("Action : {} not handled in message consumption".format(action))


class Command(BaseCommand):
    help = "Listens to a RabbitMQ topic"

    def __init__(self, callbackFn=base_callback):
        self.__callbackFn = callbackFn

    def add_arguments(self, parser):
        parser.add_argument(
            "-s",
            "--service_config_id",
            type=str,
            help="Service Config Id",
        )

    def connectToMSMQ(self, serviceConfigId):
        try:
            msmqHandler = RabbitMQ.getSingleton(serviceConfigId)
            return msmqHandler
        except Exception as e:
            print(e)
            logger.error(
                "Error connecting to RabbitMQ, check if RabbitMQ instance is up!"
            )
            Time.sleep(30)
            self.connectToMSMQ()

    def handle(self, *args, **options):
        logger.info("Listening to MSMQ messages...")
        serviceConfigId = DictUtils.get(options, "service_config_id")
        msmqHandler = self.connectToMSMQ(serviceConfigId)
        if msmqHandler.isMQMQEnabled():
            msmqHandler.consumeQueues(self.__callbackFn)
            msmqHandler.startConsuming()
        else:
            logger.info("MSMQ is disabled")



from rest_framework import status
from kfsd.apps.core.auth.api.gateway import APIGateway
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class SSO(APIGateway):
    def __init__(self, request=None):
        APIGateway.__init__(self, request)

    def getVerifyTokensUrl(self):
        return self.constructUrl(
            ["services.gateway_api.host", "services.gateway_api.sso.verify_tokens_uri"]
        )

    def verifyTokens(self, payload):
        return self.httpPost(self.getVerifyTokensUrl(), payload, status.HTTP_200_OK)

    def isAuthEnabled(self):
        isAuth = self.getDjangoRequest().findConfigs(["services.is_auth_enabled"])[0]
        return isAuth if isAuth else False



from django.db import models

from kfsd.apps.models.constants import MAX_LENGTH
from kfsd.apps.models.tables.base import BaseModel
from kfsd.apps.models.tables.route import Route


class Queue(BaseModel):
    name = models.CharField(max_length=MAX_LENGTH)
    is_declare = models.BooleanField(default=False)
    is_consume = models.BooleanField(default=True)
    declare_attrs = models.JSONField(default=dict)
    consume_attrs = models.JSONField(default=dict)
    routes = models.ManyToManyField(Route)

    def save(self, *args, **kwargs):
        self.identifier = "{}={}".format("QUEUE", self.name)
        return super().save(*args, **kwargs)

    class Meta:
        app_label = "models"
        verbose_name = "Queue"
        verbose_name_plural = "Queue"



MIN_LENGTH = 1
MAX_LENGTH = 400

IDENTIFIER_REGEX_CONDITION = r"^[A-Za-z0-9\.\-:_,=\/@]+$"
MSMQ_NAME_REGEX_CONDITION = r"^[A-Za-z0-9_\.]+$"
NAME_REGEX_CONDITION = r"^[A-Za-z0-9/\-_:,\s@\.]+$"


ORG = "ORG"
APPLICATION = "APP"
PROJECT = "PRJ"
TYPE = "TYPE"
NAMESPACE = "NS"

# SERVICE_CONFIG_ID FE
SERVICE_CONFIG_ID_SSO_FE = "sso_fe"

# SERVICE_CONFIG_ID API
SERVICE_CONFIG_ID_UTILS_API = "utils_api"
SERVICE_CONFIG_ID_SSO_API = "sso_api"
SERVICE_CONFIG_ID_GATEWAY_API = "gateway_api"
SERVICE_CONFIG_ID_CERTS_API = "certs_api"
SERVICE_CONFIG_ID_PERMS_API = "perms_api"
SERVICE_CONFIG_ID_REMIND_API = "remind_api"
SERVICE_CONFIG_ID_FILE_API = "file_api"
SERVICE_CONFIG_ID_ACCOUNTS_API = "accounts_api"
SERVICE_CONFIG_ID_BIND_API = "bind_api"
SERVICE_CONFIG_ID_ESP_GATEWAY_API = "esp_gateway_api"
SERVICE_CONFIG_ID_ESP_POSTMARK_API = "esp_postmark_api"


# COMMON ACTIONS
INBOUND_SIGNAL_CLEAR_OUTPOST = "INBOUND_SIGNAL_CLEAR_OUTPOST"



from django.db import models

from kfsd.apps.models.constants import MAX_LENGTH
from kfsd.apps.models.tables.base import BaseModel
from kfsd.apps.models.tables.exchange import Exchange


class Route(BaseModel):
    exchange = models.ForeignKey(Exchange, on_delete=models.CASCADE)
    routing_key = models.CharField(max_length=MAX_LENGTH)

    def save(self, *args, **kwargs):
        self.identifier = "{}={}".format("ROUTE", self.routing_key)
        return super().save(*args, **kwargs)

    class Meta:
        app_label = "models"
        verbose_name = "Route"
        verbose_name_plural = "Routes"



from django.urls import path, re_path
from django.views.generic.base import RedirectView
from kfsd.apps.frontend.views.docs.api import APIDocsView

urlpatterns = [
    re_path(
        r"^$", RedirectView.as_view(url="apis/doc/", permanent=False), name="api_doc"
    ),
    path("apis/doc/", APIDocsView.as_view(), name="api_doc"),
]



from drf_spectacular.utils import extend_schema_view

from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.link import Link
from kfsd.apps.endpoints.serializers.common.link import LinkViewModelSerializer
from kfsd.apps.endpoints.views.docs.link import LinkDoc


@extend_schema_view(**LinkDoc.modelviewset())
class LinkModelViewSet(CustomModelViewSet):
    queryset = Link.objects.all()
    serializer_class = LinkViewModelSerializer



from rest_framework import serializers
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
)

from kfsd.apps.endpoints.serializers.common.model import BaseModelSerializer
from kfsd.apps.models.tables.queue import Queue
from kfsd.apps.models.tables.route import Route
from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH


class QueueModelSerializer(BaseModelSerializer):
    name = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
        ],
    )
    is_declare = serializers.BooleanField(default=False)
    is_consume = serializers.BooleanField(default=False)
    declare_attrs = serializers.JSONField(default=dict)
    consume_attrs = serializers.JSONField(default=dict)
    routes = serializers.SlugRelatedField(
        many=True,
        read_only=False,
        slug_field="identifier",
        queryset=Route.objects.all(),
    )

    class Meta:
        model = Queue
        fields = "__all__"


class QueueViewModelSerializer(QueueModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = Queue
        exclude = ("created", "updated", "id")



from rest_framework import status
from django.urls import reverse

from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler


class ArrUtilsTestCases(EndpointsTestHandler):
    def setUp(self):
        super().setUp()

    def test_utils_arr_join(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/utils/arr/test_utils_arr_join.json"
        )
        obsResponse = self.post(reverse("utils-arr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/utils/arr/test_utils_arr_join.json"
        )
        self.assertEqual(obsResponse, expResponse)

    def test_utils_arr_intersection(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/utils/arr/test_utils_arr_intersection.json"
        )
        obsResponse = self.post(reverse("utils-arr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/utils/arr/test_utils_arr_intersection.json"
        )
        self.assertEqual(obsResponse, expResponse)

    def test_utils_arr_merge(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/utils/arr/test_utils_arr_merge.json"
        )
        obsResponse = self.post(reverse("utils-arr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/utils/arr/test_utils_arr_merge.json"
        )
        self.assertEqual(obsResponse, expResponse)

    def test_utils_arr_merge_uniq(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/utils/arr/test_utils_arr_merge_uniq.json"
        )
        obsResponse = self.post(reverse("utils-arr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/utils/arr/test_utils_arr_merge_uniq.json"
        )
        self.assertEqual(obsResponse, expResponse)

    def test_utils_arr_match_and_merge(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/utils/arr/test_utils_arr_match_and_merge.json"
        )
        obsResponse = self.post(reverse("utils-arr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/utils/arr/test_utils_arr_match_and_merge.json"
        )
        self.assertEqual(
            list(obsResponse["output"]["value"]), expResponse["output"]["value"]
        )



from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.exchange import Exchange
from kfsd.apps.endpoints.serializers.common.exchange import ExchangeViewModelSerializer


class ExchangeModelViewSet(CustomModelViewSet):
    queryset = Exchange.objects.all()
    serializer_class = ExchangeViewModelSerializer



from kfsd.apps.core.common.configuration import Configuration
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class DjangoConfig:
    def __init__(self, config={}):
        self.__config = config

    def getConfig(self):
        return self.__config

    def findConfigs(self, paths):
        configPaths = Configuration.findConfigValues(self.getConfig(), paths)
        if None in configPaths:
            logger.error(
                "Requested Config Paths: {}, values found: {}".format(
                    paths, configPaths
                )
            )
        return configPaths



from rest_framework import status
from django.urls import reverse

from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler


class AttrUtilsTestCases(EndpointsTestHandler):
    def setUp(self):
        super().setUp()

    def test_utils_expr(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/utils/attr/test_utils_expr.json"
        )
        obsResponse = self.post(reverse("utils-attr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/utils/attr/test_utils_expr.json"
        )
        self.assertEqual(obsResponse, expResponse)



from rest_framework import status
from django.urls import reverse

from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler


class ConfigurationTestCases(EndpointsTestHandler):
    def setUp(self):
        super().setUp()

    def postYaml(
        self,
        url,
        data,
        expStatus,
    ):
        response = self.client.post(url, data, format="yaml")
        self.assertEqual(response.status_code, expStatus)
        print(response)

    def test_common_config_dev(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/common/config/test_common_config_dev.json"
        )
        obsResponse = self.post(reverse("common-config"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/common/config/test_common_config_dev.json"
        )
        self.assertEqual(obsResponse, expResponse)

    def test_common_config_prod(self):
        postData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/requests/common/config/test_common_config_prod.json"
        )
        obsResponse = self.post(reverse("common-config"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/data/responses/common/config/test_common_config_prod.json"
        )
        self.assertEqual(obsResponse, expResponse)



from django.db import models

from kfsd.apps.models.constants import MAX_LENGTH
from kfsd.apps.models.tables.base import BaseModel


# Github, Twitter, Linkedin, Youtube, Website
class Reference(BaseModel):
    type = models.CharField(max_length=MAX_LENGTH)
    attrs = models.JSONField(default=dict)

    class Meta:
        app_label = "models"
        verbose_name = "Reference"
        verbose_name_plural = "References"



from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.producer import Producer
from kfsd.apps.endpoints.serializers.common.producer import ProducerViewModelSerializer


class ProducerModelViewSet(CustomModelViewSet):
    queryset = Producer.objects.all()
    serializer_class = ProducerViewModelSerializer



from rest_framework import serializers
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
)

from kfsd.apps.endpoints.serializers.common.model import BaseModelSerializer
from kfsd.apps.models.tables.route import Route
from kfsd.apps.models.tables.producer import Producer
from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH


class ProducerModelSerializer(BaseModelSerializer):
    signal = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
        ],
    )
    route = serializers.SlugRelatedField(
        many=False,
        read_only=False,
        slug_field="identifier",
        queryset=Route.objects.all(),
    )
    properties = serializers.JSONField(default=dict)

    class Meta:
        model = Producer
        fields = "__all__"


class ProducerViewModelSerializer(ProducerModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = Producer
        exclude = ("created", "updated", "id")



from django.urls import path, include
from drf_spectacular.views import SpectacularAPIView
from rest_framework import routers

from kfsd.apps.endpoints.views.common.exchange import ExchangeModelViewSet
from kfsd.apps.endpoints.views.common.queue import QueueModelViewSet
from kfsd.apps.endpoints.views.common.route import RouteModelViewSet
from kfsd.apps.endpoints.views.common.producer import ProducerModelViewSet

from kfsd.apps.endpoints.views.utils.utils import UtilsViewSet
from kfsd.apps.endpoints.views.utils.common import CommonViewSet
from kfsd.apps.endpoints.views.common.outpost import OutpostModelViewSet

router = routers.DefaultRouter()
router.include_format_suffixes = False

router.register("exchange", ExchangeModelViewSet, basename="exchange")
router.register("queue", QueueModelViewSet, basename="queue")
router.register("route", RouteModelViewSet, basename="route")
router.register("producer", ProducerModelViewSet, basename="producer")

router.register("utils", UtilsViewSet, basename="utils")
router.register("common", CommonViewSet, basename="common")
router.register("outpost", OutpostModelViewSet, basename="outpost")

urlpatterns = [
    path("", include(router.urls)),
    path("schema/", SpectacularAPIView.as_view(), name="schema-api"),
]



from django.db import models

from kfsd.apps.core.utils.system import System
from kfsd.apps.models.constants import MAX_LENGTH
from kfsd.apps.models.tables.base import BaseModel
from kfsd.apps.models.tables.platform import Platform


# Github, Twitter, Linkedin, Youtube, Website
class Link(BaseModel):
    link = models.TextField()
    platform = models.ForeignKey(
        Platform, on_delete=models.CASCADE, null=True, blank=True
    )
    link_id = models.CharField(max_length=MAX_LENGTH)

    def save(self, *args, **kwargs):
        self.uniq_id = System.api_key(6)
        if self.platform:
            self.identifier = "{},LINK_ID={}".format(
                self.platform.identifier, self.link_id
            )
        else:
            self.identifier = "LINK_ID={}".format(self.link_id)
        return super().save(*args, **kwargs)

    class Meta:
        app_label = "models"
        verbose_name = "Link"
        verbose_name_plural = "Link"



from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.route import Route
from kfsd.apps.endpoints.serializers.common.route import RouteViewModelSerializer


class RouteModelViewSet(CustomModelViewSet):
    queryset = Route.objects.all()
    serializer_class = RouteViewModelSerializer



from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.queue import Queue
from kfsd.apps.endpoints.serializers.common.queue import QueueViewModelSerializer


class QueueModelViewSet(CustomModelViewSet):
    queryset = Queue.objects.all()
    serializer_class = QueueViewModelSerializer



from django.db import models

from kfsd.apps.models.constants import MAX_LENGTH
from kfsd.apps.models.tables.base import BaseModel


class Exchange(BaseModel):
    name = models.CharField(max_length=MAX_LENGTH)
    attrs = models.JSONField(default=dict)

    def save(self, *args, **kwargs):
        self.identifier = "{}={}".format("EXCHANGE", self.name)
        return super().save(*args, **kwargs)

    class Meta:
        app_label = "models"
        verbose_name = "Exchange"
        verbose_name_plural = "Exchanges"



from django.core.management.base import BaseCommand

from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.file import FileUtils
from kfsd.apps.core.utils.system import System

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class Command(BaseCommand):
    help = "Dev setup"

    def add_arguments(self, parser):
        parser.add_argument(
            "-c",
            "--config",
            type=str,
            help="Settings",
        )
        parser.add_argument(
            "-d",
            "--working_dir",
            type=str,
            help="Working Dir",
        )
        parser.add_argument(
            "-u",
            "--utils",
            type=str,
            help="Is Utils Pkg",
        )
        parser.add_argument(
            "-mm",
            "--makemigration",
            type=bool,
            default=True,
            help="Make Migrations",
        )
        parser.add_argument(
            "-m",
            "--migrate",
            type=bool,
            default=True,
            help="Make Migrations",
        )

    def genPyEnv(self, utilsVersion):
        FileUtils.rm_file("db.sqlite3")
        if utilsVersion:
            self.updateUtilsPkg(utilsVersion)

    def makeMigrations(self, settingsPath, workingDir, utilsVersion):
        system = System()
        if utilsVersion:
            migrationsDir = FileUtils.construct_path(
                workingDir, "kubefacets/apps/backend/migrations"
            )
            FileUtils.rm_dir(migrationsDir)
            FileUtils.create_dir(migrationsDir)

            kfsdMigrationsDir = FileUtils.construct_path(
                workingDir,
                "py_env/lib/python3.10/site-packages/kfsd/apps/models/migrations",
            )
            FileUtils.rm_dir(kfsdMigrationsDir)
            FileUtils.create_dir(kfsdMigrationsDir)
            cmds = [
                "touch {}/__init__.py".format(migrationsDir),
                "touch {}/__init__.py".format(kfsdMigrationsDir),
            ]
            system.cmdsExec(cmds, False)
        else:
            migrationsDir = FileUtils.construct_path(
                workingDir, "kfsd/apps/models/migrations"
            )
            FileUtils.rm_dir(migrationsDir)
            FileUtils.create_dir(migrationsDir)
            cmd = "touch {}/__init__.py".format(migrationsDir)
            system.cmdExec(cmd, False)

        cmds = ["python manage.py makemigrations --settings={}".format(settingsPath)]
        system.cmdsExec(cmds, False)

    def migrate(self, settingsPath):
        system = System()
        cmds = ["python manage.py migrate --settings={}".format(settingsPath)]
        system.cmdsExec(cmds, False)

    def updateUtilsPkg(self, utilsVersion):
        filePath = "requirements.txt"
        requirements = FileUtils.read(filePath)
        requirements = [req for req in requirements if not req.startswith("kfsd")]
        requirements.append("kfsd=={}".format(utilsVersion))
        requirementsStr = "\n".join(requirements)
        FileUtils.write(filePath, requirementsStr)

    def devSetup(
        self, settingsPath, workingDir, utilsVersion, isMakeMigrations, isMigrate
    ):
        self.genPyEnv(utilsVersion)
        if isMakeMigrations:
            self.makeMigrations(settingsPath, workingDir, utilsVersion)
        if isMigrate:
            self.migrate(settingsPath)

    def handle(self, *args, **options):
        logger.info("Dev Setup...")
        workingDir = DictUtils.get(options, "working_dir")
        utilsVersion = DictUtils.get(options, "utils", None)
        isMakeMigrations = DictUtils.get(options, "makemigration")
        isMigrate = DictUtils.get(options, "migrate")
        settingsPath = DictUtils.get(options, "config")
        logger.info(
            "Recd settings_path: {}, working_dir: {}, is_utils_pkg: {}, is_make_migrations: {}, is_migrate: {}".format(
                settingsPath, workingDir, utilsVersion, isMakeMigrations, isMigrate
            )
        )
        self.devSetup(
            settingsPath, workingDir, utilsVersion, isMakeMigrations, isMigrate
        )



from rest_framework import serializers
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
)

from kfsd.apps.endpoints.serializers.common.model import BaseModelSerializer
from kfsd.apps.models.tables.exchange import Exchange
from kfsd.apps.models.tables.route import Route
from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH


class RouteModelSerializer(BaseModelSerializer):
    exchange = serializers.SlugRelatedField(
        many=False,
        read_only=False,
        slug_field="identifier",
        queryset=Exchange.objects.all(),
    )
    routing_key = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
        ],
    )

    class Meta:
        model = Route
        fields = "__all__"


class RouteViewModelSerializer(RouteModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = Route
        exclude = ("created", "updated", "id")
